import os
import shutil

import cx_Oracle as almadb
import psycopg2  as  archivedb
import sys
from events import events
from pycapo import CapoConfig

"""
ALMA reingester script for the NRAO archive.
The plan here is to connect to our archive and see what the newest ALMA 
execblock we have. Then we connect to ALMA and get all the execblocks newer than
that. Once we have the execblocks on disk, we run the ingestion system against
it.
"""

config = CapoConfig()


# Connects to the ALMA database
def connect_alma():
    user   = config['almaMetadataDatabase.jdbcUsername']
    passwd = config['almaMetadataDatabase.jdbcPassword']
    url    = config['almaMetadataDatabase.jdbcUrl']

    hostport, service_name = url.split('/')[-2:]
    host, port = hostport.split(':')
    dsn = almadb.makedsn(host, port, service_name=service_name.upper())
    return almadb.connect(user, passwd, dsn)


# Connects to the archive database
def connect_archive():
    user   = config['metadataDatabase.jdbcUsername']
    passwd = config['metadataDatabase.jdbcPassword']
    url    = str(config['metadataDatabase.jdbcUrl'])

    host, db = url.split('/')[-2:]
    return archivedb.connect('host={} dbname={} user={} password={}'.format(host, db, user, passwd))


archive_conn = connect_archive()
alma_conn = connect_alma()

# When was the last ALMA EB ingested?
def newest_alma_execblock():
    cur = archive_conn.cursor()
    cur.execute("SELECT MAX(mjd_to_timestamp(endtime)) "
                "FROM archive.public.execution_blocks "
                "WHERE telescope = 'ALMA'")
    newest_block = cur.fetchone()[0]
    print('Newest block in archive is {}'.format(newest_block))
    return newest_block


def reingest_execblock(execblockuid):
    fsname = execblockuid.replace(':', '_').replace('/', '_')

    # first run the alma-data-fetcher to obtain this data set
    fetch_execblock(execblockuid, fsname)

    # second, run ingest
    success = ingest_execblock(execblockuid, fsname)

    # no longer need to remove the execblock
    # shutil.rmtree('rawdata/' + fsname)

    # send an event about whether it worked
    events.broadcast('ALMA reingestion complete',
                     application='alma-reingester',
                     routing_key='ingestion-complete.metadata',
                     logData={'successful': success,
                              'asdmUid': execblockuid})


def ingest_execblock(execblockuid, fsname):
    command = 'ingest -r -m -d -o {} -p {}/rawdata'\
        .format(fsname, os.getcwd())
    return os.system(command) == 0


def fetch_execblock(execblockuid, fsname):
    os.system('alma-data-fetcher.sh . SDMonly {}'.format(execblockuid))


def main():
    # ensure our path includes the workflow root
    os.environ['PATH'] = '~/workflows/{}/bin'.format(os.environ['CAPO_PROFILE']) + os.pathsep + os.environ['PATH']
    reingest_execblock(sys.argv[1])
